@@ -26,7 +26,7 @@
 class ScilabBackend : public Cantor::Backend
 {
     Q_OBJECT
-    
+
     public:
         explicit ScilabBackend( QObject* parent = nullptr,const QList<QVariant> args = QList<QVariant>());
         ~ScilabBackend() override;
@@ -35,7 +35,7 @@ class ScilabBackend : public Cantor::Backend
         QString version() const override;
         Cantor::Session *createSession() override;
         Cantor::Backend::Capabilities capabilities() const override;
-        bool requirementsFullfilled() const override;
+        bool requirementsFullfilled(QString* const reason = nullptr) const override;
 
         QWidget* settingsWidget(QWidget* parent) const override;
         KConfigSkeleton* config() const override;
