@@ -22,6 +22,9 @@
 
 #include <config-cantor.h>
 
+#include <array>
+#include <cmath>
+
 #include "cantor_part.h"
 #include "lib/assistant.h"
 #include "lib/backend.h"
@@ -45,6 +48,7 @@
 #include <KRun>
 #include <KStandardAction>
 #include <KToggleAction>
+#include <KSelectAction>
 #include <KXMLGUIFactory>
 #include <KZip>
 
@@ -59,7 +63,8 @@
 #include <QTextStream>
 #include <QTextEdit>
 #include <QTimer>
-
+#include <QRegularExpression>
+#include <QComboBox>
 
 //A concrete implementation of the WorksheetAccesssInterface
 class WorksheetAccessInterfaceImpl : public Cantor::WorksheetAccessInterface
@@ -108,6 +113,7 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     m_panelHandler(new Cantor::PanelPluginHandler(this)),
     m_initProgressDlg(nullptr),
     m_showProgressDlg(true),
+    m_currectZoomAction(nullptr),
     m_showBackendHelp(nullptr),
     m_statusBarBlocked(false),
     m_sessionStatusCounter(0)
@@ -215,6 +221,7 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     KStandardAction::zoomIn(m_worksheetview, SLOT(zoomIn()), collection);
     KStandardAction::zoomOut(m_worksheetview, SLOT(zoomOut()), collection);
     KStandardAction::actualSize(m_worksheetview, SLOT(actualSize()), collection);
+    connect(m_worksheetview, &WorksheetView::scaleFactorChanged, this, &CantorPart::updateZoomWidgetValue);
 
     m_evaluate = new QAction(i18n("Evaluate Worksheet"), collection);
     collection->addAction(QLatin1String("evaluate_worksheet"), m_evaluate);
@@ -223,6 +230,22 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     connect(m_evaluate, &QAction::triggered, this, &CantorPart::evaluateOrInterrupt);
     m_editActions.push_back(m_evaluate);
 
+    //
+    m_zoom = new KSelectAction(QIcon::fromTheme(QLatin1String("page-zoom")), i18n("Zoom"), collection);
+    connect(m_zoom, static_cast<void (KSelectAction::*)(const QString&)>(&KSelectAction::triggered), this, &CantorPart::zoomValueEdited);
+    static constexpr std::array<double, 8> ZoomValues = {0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 2.0, 4.0};
+    QStringList zoomNames;
+    for (double zoomValue : ZoomValues)
+    {
+        const std::string& zoomName = std::to_string(static_cast<int>(zoomValue * 100)) + "%";
+        zoomNames << i18n(zoomName.c_str());
+    }
+    m_zoom->setItems(zoomNames);
+    m_zoom->setEditable(true);
+    Q_ASSERT(std::find(ZoomValues.begin(), ZoomValues.end(), 1.0) != ZoomValues.end());
+    m_zoom->setCurrentItem(std::distance(ZoomValues.begin(), std::find(ZoomValues.begin(), ZoomValues.end(), 1.0)));
+    collection->addAction(QLatin1String("zoom_selection_action"), m_zoom);
+
     m_typeset = new KToggleAction(i18n("Typeset using LaTeX"), collection);
     m_typeset->setChecked(Settings::self()->typesetDefault());
     // Disable until login, because we use session command for this action
@@ -308,6 +331,21 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     connect(insertImageEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertImageEntry()));
     m_editActions.push_back(insertImageEntry);
 
+    QAction* collapseAllEntries = new QAction(QIcon(), i18n("Collapse All Results"), collection);
+    collection->addAction(QLatin1String("all_entries_collapse_results"), collapseAllEntries);
+    connect(collapseAllEntries, &QAction::triggered, m_worksheet, &Worksheet::collapseAllResults);
+    m_editActions.push_back(collapseAllEntries);
+
+    QAction* uncollapseAllEntries = new QAction(QIcon(), i18n("Expand All Results"), collection);
+    collection->addAction(QLatin1String("all_entries_uncollapse_results"), uncollapseAllEntries );
+    connect(uncollapseAllEntries , &QAction::triggered, m_worksheet, &Worksheet::uncollapseAllResults);
+    m_editActions.push_back(uncollapseAllEntries);
+
+    QAction* removeAllResults = new QAction(QIcon(), i18n("Remove All Results"), collection);
+    collection->addAction(QLatin1String("all_entries_remove_all_results"), removeAllResults);
+    connect(removeAllResults, &QAction::triggered, m_worksheet, &Worksheet::removeAllResults);
+    m_editActions.push_back(removeAllResults);
+
     QAction* removeCurrent = new QAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Remove current Entry"), collection);
     collection->addAction(QLatin1String("remove_current"), removeCurrent);
     collection->setDefaultShortcut(removeCurrent, Qt::ShiftModifier + Qt::Key_Delete);
@@ -337,7 +375,7 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     QAction* showCompletion = new QAction(i18n("Show Completion"), collection);
     collection->addAction(QLatin1String("show_completion"), showCompletion);
     QList<QKeySequence> showCompletionShortcuts;
-    showCompletionShortcuts << Qt::Key_Tab << Qt::CTRL + Qt::Key_Space;
+    showCompletionShortcuts << Qt::CTRL + Qt::Key_Space; // No Tab, because the tab handeled by entries itself
     collection->setDefaultShortcuts(showCompletion, showCompletionShortcuts);
     connect(showCompletion, SIGNAL(triggered()), m_worksheet, SLOT(showCompletion()));
     m_editActions.push_back(showCompletion);
@@ -438,6 +476,8 @@ bool CantorPart::openFile()
     if (rc) {
         qDebug()<< "Worksheet successfully loaded in " <<  (float)timer.elapsed()/1000 << " seconds";
         updateCaption();
+        if (m_worksheet && m_worksheet->session() && m_worksheet->session()->backend())
+            setBackendName(m_worksheet->session()->backend()->id());
         // We modified, but it we load file now, so no need in save option
         setModified(false);
     }
@@ -996,5 +1036,34 @@ void CantorPart::showImportantStatusMessage(const QString& message)
     QTimer::singleShot(3000, this, SLOT(unblockStatusBar()));
 }
 
+void CantorPart::zoomValueEdited(const QString& text)
+{
+    static const QRegularExpression zoomRegexp(QLatin1String("(?:(\\d+)%|(\\d+))"));
+
+    QRegularExpressionMatch match = zoomRegexp.match(text);
+    if (match.hasMatch())
+    {
+        double zoom = match.captured(1).toDouble() / 100.0;
+        if (m_worksheetview)
+            m_worksheetview->setScaleFactor(zoom, false);
+    }
+}
+
+void CantorPart::updateZoomWidgetValue(double zoom)
+{
+    if (m_zoom)
+    {
+        double scale = zoom;
+        scale = round(scale * 100);
+        const QString& searchText = QString::number((int)scale) + QLatin1String("%");
+        if (m_currectZoomAction)
+            m_currectZoomAction->setText(searchText);
+        else
+            m_currectZoomAction = m_zoom->addAction(searchText);
+        m_zoom->setCurrentAction(m_currectZoomAction);
+    }
+}
+
+
 K_PLUGIN_FACTORY_WITH_JSON(CantorPartFactory, "cantor_part.json", registerPlugin<CantorPart>();)
 #include "cantor_part.moc"
