@@ -49,6 +49,7 @@
 #include "imageentry.h"
 #include "pagebreakentry.h"
 #include "placeholderentry.h"
+#include "horizontalruleentry.h"
 #include "lib/jupyterutils.h"
 #include "lib/backend.h"
 #include "lib/extension.h"
@@ -214,11 +215,19 @@ void Worksheet::updateLayout()
         cursorRectVisible = worksheetView()->isVisible(cursorRect);
     }
 
+    qreal maxPromptWidth = 0;
+    if (Settings::useOldCantorEntriesIndent() == false)
+    {
+        for (WorksheetEntry *entry = firstEntry(); entry; entry = entry->next())
+            if (entry->type() == CommandEntry::Type)
+                maxPromptWidth = std::max(static_cast<CommandEntry*>(entry)->promptItemWidth(), maxPromptWidth);
+    }
+
     const qreal w = m_viewWidth - LeftMargin - RightMargin;
     qreal y = TopMargin;
     const qreal x = LeftMargin;
     for (WorksheetEntry *entry = firstEntry(); entry; entry = entry->next())
-        y += entry->setGeometry(x, y, w);
+        y += entry->setGeometry(x, x+maxPromptWidth, y, w);
 
     setSceneRect(QRectF(0, 0, sceneRect().width(), y));
     if (cursorRectVisible)
@@ -610,6 +619,12 @@ void Worksheet::appendCommandEntry(const QString& text)
     }
 }
 
+WorksheetEntry * Worksheet::appendHorizontalRuleEntry()
+{
+    return appendEntry(HorizontalRuleEntry::Type);
+}
+
+
 WorksheetEntry* Worksheet::insertEntry(const int type, WorksheetEntry* current)
 {
     if (!current)
@@ -682,6 +697,11 @@ void Worksheet::insertCommandEntry(const QString& text)
     }
 }
 
+WorksheetEntry * Worksheet::insertHorizontalRuleEntry(WorksheetEntry* current)
+{
+    return insertEntry(HorizontalRuleEntry::Type, current);
+}
+
 WorksheetEntry* Worksheet::insertEntryBefore(int type, WorksheetEntry* current)
 {
     if (!current)
@@ -743,6 +763,11 @@ WorksheetEntry* Worksheet::insertLatexEntryBefore(WorksheetEntry* current)
     return insertEntryBefore(LatexEntry::Type, current);
 }
 
+WorksheetEntry * Worksheet::insertHorizontalRuleEntryBefore(WorksheetEntry* current)
+{
+    return insertEntryBefore(HorizontalRuleEntry::Type, current);
+}
+
 void Worksheet::interrupt()
 {
     if (m_session->status() == Cantor::Session::Running)
@@ -912,6 +937,8 @@ void Worksheet::enableExpressionNumbering(bool enable)
 {
     m_showExpressionIds=enable;
     emit updatePrompt();
+    if (views().size() != 0)
+        updateLayout();
 }
 
 QDomDocument Worksheet::toXML(KZip* archive)
@@ -1271,6 +1298,11 @@ bool Worksheet::loadCantorWorksheet(const KZip& archive)
             entry = appendEntry(ImageEntry::Type, false);
             entry->setContent(expressionChild, archive);
         }
+        else if (tag == QLatin1String("HorizontalRule"))
+        {
+            entry = appendEntry(HorizontalRuleEntry::Type, false);
+            entry->setContent(expressionChild, archive);
+        }
 
         if (m_readOnly && entry)
         {
@@ -1465,6 +1497,11 @@ bool Worksheet::loadJupyterNotebook(const QJsonDocument& doc)
                 entry = appendEntry(TextEntry::Type, false);
                 entry->setContentFromJupyter(cell);
             }
+            else if (HorizontalRuleEntry::isConvertableToHorizontalRuleEntry(cell))
+            {
+                entry = appendEntry(HorizontalRuleEntry::Type, false);
+                entry->setContentFromJupyter(cell);
+            }
             else
             {
                 entry = appendEntry(MarkdownEntry::Type, false);
@@ -1602,6 +1639,7 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
     #endif
             convertTo->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Image"), entry, &WorksheetEntry::convertToImageEntry);
             convertTo->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Page Break"), entry, &WorksheetEntry::converToPageBreakEntry);
+            convertTo->addAction(QIcon(), i18n("Horizontal Line"), entry, &WorksheetEntry::convertToHorizontalRuleEntry);
 
             insert->addAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Command Entry"), entry, SLOT(insertCommandEntry()));
             insert->addAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Text Entry"), entry, SLOT(insertTextEntry()));
@@ -1613,6 +1651,7 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
     #endif
             insert->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Image"), entry, SLOT(insertImageEntry()));
             insert->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Page Break"), entry, SLOT(insertPageBreakEntry()));
+            insert->addAction(QIcon(), i18n("Horizontal Line"), entry, SLOT(insertHorizontalRuleEntry()));
 
             insertBefore->addAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Command Entry"), entry, SLOT(insertCommandEntryBefore()));
             insertBefore->addAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Text Entry"), entry, SLOT(insertTextEntryBefore()));
@@ -1624,6 +1663,7 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
     #endif
             insertBefore->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Image"), entry, SLOT(insertImageEntryBefore()));
             insertBefore->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Page Break"), entry, SLOT(insertPageBreakEntryBefore()));
+            insertBefore->addAction(QIcon(), i18n("Horizontal Line"), entry, SLOT(insertHorizontalRuleEntryBefore()));
 
             convertTo->setTitle(i18n("Convert Entry To"));
             convertTo->setIcon(QIcon::fromTheme(QLatin1String("gtk-convert")));
@@ -1646,6 +1686,7 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
     #endif
             menu->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Insert Image"), this, SLOT(appendImageEntry()));
             menu->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Insert Page Break"), this, SLOT(appendPageBreakEntry()));
+            menu->addAction(QIcon(), i18n("Insert Horizontal Line"), this, &Worksheet::appendHorizontalRuleEntry);
         }
     }
     else
@@ -1655,6 +1696,24 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
         menu->addAction(QIcon::fromTheme(QLatin1String("go-down")), i18n("Move Entries Down"), this, SLOT(selectionMoveDown()), 0);
         menu->addAction(QIcon::fromTheme(QLatin1String("media-playback-start")), i18n("Evaluate Entries"), this, SLOT(selectionEvaluate()), 0);
         menu->addAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Remove Entries"), this, SLOT(selectionRemove()), 0);
+
+        bool isAnyCommandEntryInSelection = false;
+        for (WorksheetEntry* entry : m_selectedEntries)
+            if (entry->type() == CommandEntry::Type)
+            {
+                isAnyCommandEntryInSelection = true;
+                break;
+            }
+
+        if (isAnyCommandEntryInSelection)
+        {
+            menu->addSeparator();
+            menu->addAction(QIcon(), i18n("Collapse Command Entry Results"), this, &Worksheet::collapseSelectionResults);
+            menu->addAction(QIcon(), i18n("Expand Command Entry Results"), this, &Worksheet::uncollapseSelectionResults);
+            menu->addAction(QIcon(), i18n("Remove Command Entry Results"), this, &Worksheet::removeSelectionResults);
+            menu->addAction(QIcon(), i18n("Exclude Command Entry From Execution"), this, &Worksheet::excludeFromExecutionSelection);
+            menu->addAction(QIcon(), i18n("Add Command Entry To Execution"), this, &Worksheet::addToExectuionSelection);
+        }
     }
 }
 
@@ -2510,3 +2569,79 @@ void Worksheet::notifyEntryFocus(WorksheetEntry* entry)
     else
         m_circularFocusBuffer.clear();
 }
+
+void Worksheet::collapseAllResults()
+{
+    for (WorksheetEntry *entry = firstEntry(); entry; entry = entry->next())
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->collapseResults();
+}
+
+void Worksheet::uncollapseAllResults()
+{
+    for (WorksheetEntry *entry = firstEntry(); entry; entry = entry->next())
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->expandResults();
+}
+
+void Worksheet::removeAllResults()
+{
+    bool remove = false;
+
+    if (KMessageBox::shouldBeShownContinue(QLatin1String("WarnAboutAllResultsRemoving")))
+    {
+        KMessageBox::ButtonCode btn = KMessageBox::warningContinueCancel(
+            views().first(),
+            i18n("This action will remove all results without the possibility of cancellation. Are you sure?"),
+            i18n("Remove all results"),
+            KStandardGuiItem::cont(),
+            KStandardGuiItem::cancel(),
+            QLatin1String("WarnAboutAllResultsRemoving")
+        );
+        remove = (btn == KMessageBox::Continue);
+    }
+    else
+        remove = true;
+
+    if (remove)
+    {
+        for (WorksheetEntry *entry = firstEntry(); entry; entry = entry->next())
+            if (entry->type() == CommandEntry::Type)
+                static_cast<CommandEntry*>(entry)->removeResults();
+    }
+}
+
+void Worksheet::addToExectuionSelection()
+{
+    for (WorksheetEntry* entry : m_selectedEntries)
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->addToExecution();
+}
+
+void Worksheet::excludeFromExecutionSelection()
+{
+    for (WorksheetEntry* entry : m_selectedEntries)
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->excludeFromExecution();
+}
+
+void Worksheet::collapseSelectionResults()
+{
+    for (WorksheetEntry* entry : m_selectedEntries)
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->collapseResults();
+}
+
+void Worksheet::uncollapseSelectionResults()
+{
+    for (WorksheetEntry* entry : m_selectedEntries)
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->expandResults();
+}
+
+void Worksheet::removeSelectionResults()
+{
+    for (WorksheetEntry* entry : m_selectedEntries)
+        if (entry->type() == CommandEntry::Type)
+            static_cast<CommandEntry*>(entry)->removeResults();
+}
